<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\Product;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Zend_Db;

/**
 * Class RatingProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class RatingProvider implements EntityProviderInterface
{
    const XML_CONFIG_REVIEW_ENABLED = "catalog/review/active";

    /** @var null */
    protected $ids;

    /** @var int */
    protected $website;

    /** @var array */
    protected $items = [];

    /** @var Manager */
    protected $moduleManager;

    /** @var ResourceConnection */
    private $resourceConnection;

    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var array
     */
    private $lockupTemplate = [];
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var
     */
    private $store;

    /**
     * StockProvider constructor.
     * @param Manager $moduleManager
     * @param ObjectManagerInterface $objectManager
     * @param ResourceConnection $resourceConnection
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Manager $moduleManager,
        ObjectManagerInterface $objectManager,
        ResourceConnection $resourceConnection,
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->moduleManager = $moduleManager;
        $this->objectManager = $objectManager;
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @return mixed
     */
    public function current()
    {
        $current = current($this->items);
        $product_id=((int)key($this->items));
        $product = new Product();
        $product->setId($product_id);
        if(isset($current["removethis"])){
            unset($current["removethis"]);
        }
        $rating = array_merge($this->lockupTemplate, $current);
        $product->setData(Product::RATING, $rating);

        return $product;
    }

    /**
     * @param $name
     * @return string|string[]
     */
    public function normalizeName($name)
    {
        $name = strtolower($name);
        $name = str_replace(" ", "_", $name);
        return $name;
    }

    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    public function key()
    {
        return current($this->items)['entity_pk_value'];
    }

    public function valid()
    {
        return current($this->items);
    }

    /**
     *
     */
    public function rewind()
    {
        if (!$this->scopeConfig->getValue(self::XML_CONFIG_REVIEW_ENABLED, ScopeInterface::SCOPE_STORE, $this->store)) {
            return false;
        }

        $ratingTable = $this->resourceConnection->getTableName("rating");
        $ratingOptionVoteAggregated = $this->resourceConnection->getTableName("rating_option_vote_aggregated");
        $catalogProductEntity = $this->resourceConnection->getTableName("catalog_product_entity");

        $sqlRating = "SELECT
                    rating.rating_code
                FROM
                    {$ratingTable}
                WHERE
                    rating.is_active = 1";

        $sql = "SELECT
                    IFNULL(rova.entity_pk_value,cpe.entity_id) as entity_pk_value,
                    sum(rova.vote_count) as vote_count,
                    ROUND(avg(rova.percent_approved),0) as percent_approved,
                    IFNULL(rova.store_id,{$this->store}) as store_id,
                    'global'as rating_code,
                    cpe.sku
                FROM
                    {$catalogProductEntity} cpe
                LEFT JOIN
                    {$ratingOptionVoteAggregated} rova ON
                    cpe.entity_id = rova.entity_pk_value AND rova.store_id = {$this->store}
                LEFT JOIN {$ratingTable} r ON
                    rova.rating_id = r.rating_id AND r.is_active = 1
                GROUP BY cpe.entity_id";

//        $lockupTemplate = $this->resourceConnection
//            ->getConnection()
//            ->fetchAll($sqlRating, [], Zend_Db::FETCH_ASSOC);
//
//        foreach ($lockupTemplate as $template) {
//            $this->lockupTemplate[$this->normalizeName($template["rating_code"])] = [
//                'review_count'   => 0,
//                'review_summary' => 0
//            ];
//        }
//
//        $sql = "SELECT
//                    IFNULL(rova.entity_pk_value,cpe.entity_id) as entity_pk_value,
//                    IFNULL(rova.vote_count,0) as vote_count,
//                    IFNULL(rova.percent_approved,0) as percent_approved,
//                    IFNULL(rova.store_id,{$this->store}) as store_id,
//                    rova.rating_id,
//                    r.rating_code,
//                    cpe.sku
//                FROM
//                    {$catalogProductEntity} cpe
//                LEFT JOIN
//                    {$ratingOptionVoteAggregated} rova ON
//                    cpe.entity_id = rova.entity_pk_value AND rova.store_id = {$this->store}
//                LEFT JOIN {$ratingTable} r ON
//                    rova.rating_id = r.rating_id AND r.is_active = 1";

        // TODO: add filter per store e per id

        $items = $this->resourceConnection
            ->getConnection()
            ->fetchAll($sql, [], Zend_Db::FETCH_ASSOC);

        foreach ($items as $item) {
            if($item["rating_code"]!=null){
                $this->items[$item['entity_pk_value']][$this->normalizeName($item["rating_code"])] = [
                    'review_count'   => (int)$item["vote_count"],
                    'review_summary' => (int)$item["percent_approved"],
                ];
            }
            else {
                $this->items[$item['entity_pk_value']]["removethis"] = [];
            }
        }
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->store = $store->getId();
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return $this|EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        throw new LogicException('Count not filter by website');
    }
}
